@@ -1,5 +1,5 @@
 /*
- * $Id: libcitadel.h 7529 2009-06-02 21:06:29Z ajc $
+ * $Id: libcitadel.h 8127 2009-12-16 21:27:59Z dothebart $
  *
  */
 
@@ -15,7 +15,7 @@
 #include <time.h>
 #include <stdlib.h>
 #include <stdarg.h>
-#define LIBCITADEL_VERSION_NUMBER	751
+#define LIBCITADEL_VERSION_NUMBER	770
 
 /*
  * Here's a bunch of stupid magic to make the MIME parser portable.
@@ -65,19 +65,25 @@ enum LogLevel {
 #define IsEmptyStr(a) ((a)[0] == '\0')
 #endif
 
+/*
+ * another word to indicate n/a for a pointer if NULL already has a "meaning"
+ */
+extern const char *StrBufNOTNULL;
+
 /*
  * Misc declarations
  */
 
 char *libcitadel_version_string(void);
 int libcitadel_version_number(void);
+void StartLibCitadel(size_t basesize);
 void ShutDownLibCitadel(void);
 
 /*
  * MIME parser declarations
  */
 
-void extract_key(char *target, char *source, char *key);
+void extract_key(char *target, char *source, long sourcelen, char *key, long keylen);
 
 void mime_parser(char *content_start, char *content_end,
 		void (*CallBack)
@@ -216,17 +222,22 @@ typedef struct _ConstStr {
 	const char *Key;
 	long len;
 }ConstStr;
-
+#define CKEY(a) (a).Key, (a).len
 
 StrBuf* NewStrBuf(void);
 StrBuf* NewStrBufDup(const StrBuf *CopyMe);
 StrBuf* NewStrBufPlain(const char* ptr, int nChars);
+long StrBufShrinkToFit(StrBuf *Buf, int Force);
+void ReAdjustEmptyBuf(StrBuf *Buf, long ThreshHold, long NewSize);
+
 int StrBufPlain(StrBuf *Buf, const char* ptr, int nChars);
 StrBuf* _NewConstStrBuf(const char* StringConstant, size_t SizeOfStrConstant);
 #define NewConstStrBuf(a) _NewConstStrBuf(a, sizeof(a))
 void FreeStrBuf (StrBuf **FreeMe);
+char *SmashStrBuf (StrBuf **SmashMe);
 void HFreeStrBuf (void *VFreeMe);
 int FlushStrBuf(StrBuf *buf);
+int FLUSHStrBuf(StrBuf *buf); /* expensive but doesn't leave content behind for others to find in case of errors */
 
 const char *ChrPtr(const StrBuf *Str);
 int StrLength(const StrBuf *Str);
@@ -235,12 +246,29 @@ long StrBufPeek(StrBuf *Buf, const char* ptr, long nThChar, char PeekValue);
 
 int StrBufTCP_read_line(StrBuf *buf, int *fd, int append, const char **Error);
 int StrBufReadBLOB(StrBuf *Buf, int *fd, int append, long nBytes, const char **Error);
+#define NNN_TERM 1
+#define O_TERM 0
+int StrBufReadBLOBBuffered(StrBuf *Buf, 
+			   StrBuf *IOBuf, 
+			   const char **BufPos,
+			   int *fd, 
+			   int append, 
+			   long nBytes, 
+			   int check, 
+			   const char **Error);
 int StrBufTCP_read_buffered_line(StrBuf *Line, 
 				 StrBuf *buf, 
 				 int *fd, 
 				 int timeout, 
 				 int selectresolution, 
 				 const char **Error);
+int StrBufTCP_read_buffered_line_fast(StrBuf *Line, 
+				      StrBuf *buf, 
+				      const char **Pos,
+				      int *fd, 
+				      int timeout, 
+				      int selectresolution, 
+				      const char **Error);
 
 int StrBufSipLine(StrBuf *LineBuf, StrBuf *Buf, const char **Ptr);
 int StrBufExtract_token(StrBuf *dest, const StrBuf *Source, int parmnum, char separator);
@@ -252,6 +280,7 @@ int StrBufExtract_int(const StrBuf* Source, int parmnum, char separator);
 int StrBufNum_tokens(const StrBuf *source, char tok);
 int StrBufRemove_token(StrBuf *Source, int parmnum, char separator);
 
+int StrBufHaveNextToken(const StrBuf *Source, const char **pStart);
 int StrBufExtract_NextToken(StrBuf *dest, const StrBuf *Source, const char **pStart, char separator);
 int StrBufSkip_NTokenS(const StrBuf *Source, const char **pStart, char separator, int nTokens);
 unsigned long StrBufExtractNext_unsigned_long(const StrBuf* Source, const char **pStart, char separator);
@@ -261,6 +290,7 @@ int StrBufExtractNext_int(const StrBuf* Source, const char **pStart, char separa
 
 void StrBufAppendBufPlain(StrBuf *Buf, const char *AppendBuf, long AppendSize, unsigned long Offset);
 void StrBufAppendBuf(StrBuf *Buf, const StrBuf *AppendBuf, unsigned long Offset);
+size_t CurlFillStrBuf_callback(void *ptr, size_t size, size_t nmemb, void *stream);
 void StrBufAppendPrintf(StrBuf *Buf, const char *format, ...);
 #ifdef SHOW_ME_VAPPEND_PRINTF
 /* so owe don't create an include depndency, this is just visible on demand. */
@@ -273,6 +303,7 @@ void StrBufCutAt(StrBuf *Buf, int AfternChars, const char *At);
 void StrBufTrim(StrBuf *Buf);
 void StrBufUpCase(StrBuf *Buf);
 void StrBufLowerCase(StrBuf *Buf);
+void StrBufStripSlashes(StrBuf *Dir, int RemoveTrailingSlash);
 void StrBufEUid_unescapize(StrBuf *target, const StrBuf *source);
 void StrBufEUid_escapize(StrBuf *target, const StrBuf *source);
 
@@ -283,6 +314,7 @@ void StrBufConvert(StrBuf *ConvertBuf, StrBuf *TmpBuf, void *pic);
 void ctdl_iconv_open(const char *tocode, const char *fromcode, void *pic);
 void StrBuf_RFC822_to_Utf8(StrBuf *Target, const StrBuf *DecodeMe, const StrBuf* DefaultCharset, StrBuf *FoundCharset);
 int StrBufDecodeBase64(StrBuf *Buf);
+int StrBufDecodeHex(StrBuf *Buf);
 int StrBufRFC2047encode(StrBuf **target, const StrBuf *source);
 int StrBufSanitizeAscii(StrBuf *Buf, const char Mute);
 #define LB			(1)		/* Internal escape chars */
@@ -291,7 +323,9 @@ int StrBufSanitizeAscii(StrBuf *Buf, const char Mute);
 void StrBufUrlescAppend(StrBuf *OutBuf, const StrBuf *In, const char *PlainIn);
 long StrEscAppend(StrBuf *Target, const StrBuf *Source, const char *PlainIn, int nbsp, int nolinebreaks);
 long StrECMAEscAppend(StrBuf *Target, const StrBuf *Source, const char *PlainIn);
-void StrMsgEscAppend(StrBuf *Target, StrBuf *Source, const char *PlainIn);
+long StrHtmlEcmaEscAppend(StrBuf *Target, const StrBuf *Source, const char *PlainIn, int nbsp, int nolinebreaks);
+void StrMsgEscAppend(StrBuf *Target, const StrBuf *Source, const char *PlainIn);
+void StrIcalEscAppend(StrBuf *Target, const StrBuf *Source, const char *PlainIn);
 
 long StrTol(const StrBuf *Buf);
 int StrToi(const StrBuf *Buf);
@@ -311,11 +345,6 @@ const char *GetIconFilename(char *MimeType, size_t len);
 
 /* tools */
 
-int Ctdl_IsUtf8SequenceStart(char Char);
-int Ctdl_GetUtf8SequenceLength(char Char);
-int Ctdl_Utf8StrLen(char *str);
-char *Ctdl_Utf8StrCut(char *str, int maxlen);
-
 
 int safestrncpy(char *dest, const char *src, size_t n);
 int num_tokens (const char *source, char tok);
@@ -329,7 +358,8 @@ size_t CtdlEncodeBase64(char *dest, const char *source, size_t sourcelen, int li
 int CtdlDecodeBase64(char *dest, const char *source, size_t length);
 unsigned int decode_hex(char *Source);
 int CtdlDecodeQuotedPrintable(char *decoded, char *encoded, int sourcelen);
-void striplt(char *);
+void StripSlashes(char *Dir, int TrailingSlash);
+size_t striplt(char *);
 int haschar(const char *st, int ch);
 void remove_token(char *source, int parmnum, char separator);
 void fmt_date(char *buf, size_t n, time_t thetime, int seconds);
@@ -346,6 +376,7 @@ char *CtdlTempFileName(char *prefix1, int prefix2);
 FILE *CtdlTempFile(void);
 void generate_uuid(char *buf);
 char *bmstrcasestr(char *text, char *pattern);
+char *bmstrcasestr_len(char *text, size_t textlen, char *pattern, size_t patlen);
 void CtdlMakeTempFileName(char *name, int len);
 char *rfc2047encode(char *line, long length);
 int is_msg_in_mset(const char *mset, long msgnum);
@@ -373,7 +404,9 @@ struct vCard {
 
 struct vCard *vcard_new(void);
 void vcard_add_prop(struct vCard *v, char *propname, char *propvalue);
-struct vCard *vcard_load(char *);
+struct vCard *vcard_load(char *vtext);
+struct vCard *VCardLoad(StrBuf *vbtext);
+
 void vcard_free(struct vCard *);
 void vcard_set_prop(struct vCard *v, char *name, char *value, int append);
 char *vcard_get_prop(struct vCard *v, char *propname, int is_partial,
@@ -387,6 +420,7 @@ long StrBufUnescape(StrBuf *Buf, int StripBlanks);
  * Hash list implementation for Citadel
  */
 #define HKEY(a) a, (sizeof(a) - 1)
+
 typedef struct HashList HashList;
 
 typedef struct HashKey HashKey;
@@ -401,7 +435,7 @@ typedef void (*TransitionFunc) (void *Item1, void *Item2, int Odd);
 typedef void (*PrintHashDataFunc) (const char *Key, void *Item, int Odd);
 
 int Flathash(const char *str, long len);
-#define IKEY(a) (const char*)(&((int)a)), sizeof(int)
+#define IKEY(a) (const char*) &a, sizeof(a)
 
 HashList *NewHash(int Uniq, HashFunc F);
 void DeleteHash(HashList **Hash);
@@ -413,7 +447,7 @@ int GetHashKeys(HashList *Hash, char ***List);
 int dbg_PrintHash(HashList *Hash, PrintHashContent first, PrintHashContent Second);
 int PrintHash(HashList *Hash, TransitionFunc Trans, PrintHashDataFunc PrintEntry);
 HashPos *GetNewHashPos(HashList *Hash, int StepWidth);
-int GetHashPosCounter(HashPos *At);
+int GetHashPosCounter(HashList *Hash, HashPos *At);
 void DeleteHashPos(HashPos **DelMe);
 int GetNextHashPos(HashList *Hash, HashPos *At, long *HKLen, const char **HashKey, void **Data);
 int GetHashAt(HashList *Hash,long At, long *HKLen, const char **HashKey, void **Data);
@@ -424,6 +458,7 @@ const void *GetSearchPayload(const void *HashVoid);
 void SortByPayload(HashList *Hash, CompareFunc SortBy);
 void generic_free_handler(void *ptr);
 void reference_free_handler(void *ptr);
+int HashLittle(const void *key, size_t length);
 
 
 void convert_spaces_to_underscores(char *str);
